CBL CICS('COBOL3') APOST
      *****************************************************************
      *
      *  MODULE NAME = BASIC310
      *
      *  DESCRIPTIVE NAME = HTTP Client Basic Authentication program
      *
      *  Licensed Materials - Property of IBM
      *
      *  SupportPac CA8J
      *
      *  (C) Copyright IBM Corporation 2006    All Rights Reserved.
      *
      *  US Government Users Restricted Rights - Use, duplication
      *  or disclosure restricted by GSA ADP Schedule Contract
      *  with IBM Corporation
      *
      *  FUNCTION =
      *     This program prepares and sends an HTTP
      *     Basic Authentication header.
      *     It generates the header by calling HTTPAUTH, and then
      *     sends it using EXEC CICS WRITE HTTPHEADER.
      *
      *     This sample requires CICS Transaction Server 3.1 or later.
      *
      *  NOTES :
      *   DEPENDENCIES = S/390
      *   RESTRICTIONS =
      *   PATCH LABEL = None
      *   MODULE TYPE = Executable
      *   PROCESSOR = COBOL
      *   ATTRIBUTES = Read only, Serially Reusable
      *
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID.    BASIC310.
       DATE-WRITTEN.  December 2005.
       DATE-COMPILED. December 2005.
      ******************************************************************
      ****                                                          ****
      ****        B A S I C   A U T H E N T I C A T I O N           ****
      ****                                                          ****
      ****                                                          ****
      ****  This program uses Assembler subroutines to create       ****
      ****  the contents of an HTTP Basic Authentication header,    ****
      ****  and then uses CICS Web Support API to add it to         ****
      ****  the headers for an outbound HTTP request.               ****
      ****                                                          ****
      ******************************************************************

       ENVIRONMENT DIVISION.

       DATA DIVISION.

       WORKING-STORAGE SECTION.

        77 WEB-RESPONSE-PTR      USAGE IS POINTER.
        77 HTTP-RESPONSE-PTR     USAGE IS POINTER.
        77 HTTP-RESPONSE-LENGTH  PIC S9(8) COMP-5.
        77 HTTP-STATUS-CODE      PIC 9999  COMP-5.
        77 HTTP-SESSION          PIC X(8)  VALUE LOW-VALUES.
        77 HTTP-STATUS-TEXT      PIC X(80).
        77 HTTP-MEDIATYPE        PIC X(56).

        01 REMOTE-HTTP-SERVER.
           02 HTTP-PORT    COMP-5    PIC S9(8) VALUE 5053.
           02 HTTP-SCHEME  COMP-5    PIC S9(8) VALUE DFHVALUE(HTTP).
           02 HTTP-HOST    PIC X(15) VALUE 'www.example.com'.
           02 HTTP-PATH    PIC X(11) VALUE '/index.html'.

        01 HTTPAUTH-PARAMETERS.
           02 HTTPAUTH-FUNCTION        COMP-5 PIC S9(8) VALUE IS 1.
           02 HTTPAUTH-USERID-LENGTH   COMP-5 PIC S9(8) VALUE IS 7.
           02 HTTPAUTH-PASSWORD-LENGTH COMP-5 PIC S9(8) VALUE IS 8.
           02 HTTPAUTH-USERID          PIC X(8) VALUE 'UUUUUUUU'.
           02 HTTPAUTH-PASSWORD        PIC X(8) VALUE 'PPPPPPPP'.

        01 BASIC-AUTHENTICATION-HEADER.
           02 BASIC-AUTH-NAME  PIC X(13)  VALUE 'Authorization'.
           02 BASIC-AUTH-VALUE PIC X(256) VALUE ALL '-'.
           02 BASIC-AUTH-VALUE-LENGTH COMP-5 PIC S9(8) VALUE ZERO.

        01 HTTP-ERROR-RESPONSE.
           02 FILLER    PIC X(19) VALUE 'WEB RECEIVE failed.'.
           02 FILLER    PIC X(9)  VALUE ' EIBRESP='.
           02 ERR-RESP  PIC 999.
           02 FILLER    PIC X(10) VALUE ' EIBRESP2='.
           02 ERR-REAS  PIC 999.
           02 FILLER    PIC X(13) VALUE ' HTTP STATUS='.
           02 HTTP-STAT PIC 999.

       LINKAGE SECTION.
        01 HTTP-RESPONSE.
           02 FILLER PIC X OCCURS 1 TO 32767
                           DEPENDING ON HTTP-RESPONSE-LENGTH.

        01 WEB-RESPONSE.
           02 FILLER PIC X OCCURS 1 TO 32767
                           DEPENDING ON HTTP-RESPONSE-LENGTH.

       PROCEDURE DIVISION.
       MAINLINE SECTION.

      * Open an HTTP session with a remote host
           EXEC CICS WEB OPEN SESSTOKEN(HTTP-SESSION)
                         SCHEME(HTTP-SCHEME)
                         HOST(HTTP-HOST)
                         HOSTLENGTH(LENGTH OF HTTP-HOST)
                         PORTNUMBER(HTTP-PORT)
                         NOHANDLE
                         END-EXEC.

      * Set up the Basic Authentication header value
           MOVE LENGTH OF BASIC-AUTH-VALUE TO BASIC-AUTH-VALUE-LENGTH.
           CALL 'HTTPAUTH' USING HTTPAUTH-FUNCTION
                           BASIC-AUTH-VALUE
                           BASIC-AUTH-VALUE-LENGTH
                           HTTPAUTH-USERID
                           HTTPAUTH-USERID-LENGTH
                           HTTPAUTH-PASSWORD
                           HTTPAUTH-PASSWORD-LENGTH.

      * Write the Basic Authentication header
           EXEC CICS WEB WRITE SESSTOKEN(HTTP-SESSION)
                         HTTPHEADER(BASIC-AUTH-NAME)
                         VALUE(BASIC-AUTH-VALUE)
                         VALUELENGTH(BASIC-AUTH-VALUE-LENGTH)
                         END-EXEC.

      * Send an HTTP GET request to the server
           EXEC CICS WEB SEND SESSTOKEN(HTTP-SESSION)
                         GET CLOSE
                         PATH(HTTP-PATH)
                         PATHLENGTH(LENGTH OF HTTP-PATH)
                         END-EXEC.

      * Read the response
           EXEC CICS WEB RECEIVE SESSTOKEN(HTTP-SESSION)
                         SET(WEB-RESPONSE-PTR)
                         LENGTH(HTTP-RESPONSE-LENGTH)
                         STATUSCODE(HTTP-STATUS-CODE)
                         STATUSTEXT(HTTP-STATUS-TEXT)
                         MEDIATYPE(HTTP-MEDIATYPE)
                         NOHANDLE
                         END-EXEC.

           IF EIBRESP EQUAL ZERO AND HTTP-STATUS-CODE EQUAL 200 THEN
              PERFORM COPY-HTTP-RESPONSE
           ELSE
              PERFORM SET-HTTP-ERROR-RESPONSE
           END-IF.

      *
      * Display up to 24 lines of the response
      *
           IF HTTP-RESPONSE-LENGTH > 1920 THEN
              MOVE 1920 TO HTTP-RESPONSE-LENGTH.

           EXEC CICS SEND ERASE FROM(HTTP-RESPONSE)
                                FLENGTH(HTTP-RESPONSE-LENGTH)
                                END-EXEC.
      *
      * Return
      *
           EXEC CICS RETURN
           END-EXEC.

      *****************************************************************
      *
      * Subroutine to copy response and remove non-display characters.
      *
      *****************************************************************
       COPY-HTTP-RESPONSE.
           EXEC CICS GETMAIN SET(HTTP-RESPONSE-PTR)
                             FLENGTH(HTTP-RESPONSE-LENGTH)
                             END-EXEC.

           SET ADDRESS OF WEB-RESPONSE  TO WEB-RESPONSE-PTR.
           SET ADDRESS OF HTTP-RESPONSE TO HTTP-RESPONSE-PTR.
           MOVE WEB-RESPONSE TO HTTP-RESPONSE.
           INSPECT HTTP-RESPONSE
                   CONVERTING X'0D251505' TO X'1E1E1E40'.
      *****************************************************************
      *
      *  Subroutine to create an error response.
      *
      *****************************************************************
       SET-HTTP-ERROR-RESPONSE.
           MOVE EIBRESP          TO ERR-RESP.
           MOVE EIBRESP2         TO ERR-REAS.
           MOVE HTTP-STATUS-CODE TO HTTP-STAT.
           MOVE LENGTH OF HTTP-ERROR-RESPONSE TO HTTP-RESPONSE-LENGTH.
           EXEC CICS GETMAIN SET(HTTP-RESPONSE-PTR)
                             FLENGTH(HTTP-RESPONSE-LENGTH)
                             END-EXEC.
           SET ADDRESS OF HTTP-RESPONSE TO HTTP-RESPONSE-PTR.
           MOVE HTTP-ERROR-RESPONSE TO HTTP-RESPONSE.